% subroutine to compute mean spike train using the MM algorithm

% Reference:
% Wu, W. and Srivastava, A., "An information-geometric framework for 
% statistical inferences in the neural spike train space", Journal 
% of Computational Neuroscience, vol. 31, pp. 725-748, 2011.


% S: a sequence of spike trains
% T: length of time domain
% means: the estimated mean spike train

function means = MM_sub(S, T)

N = length(S);              % number of trains
for i = 1:N
    nS(i) = length(S{i});    % number of spikes in each train
end

% compute the number spikes in the mean spike train
nm = ceil(median(nS));

% initialize the mean spike train as uniform on [0, T]
rand('seed', 0);
s0 = T*sort(rand(1,nm));      % random initial
s = s0;
ss = s;

hmax = 10;
for h = 1:hmax
    disp(sprintf('h = %d', h));
    if h == hmax
        disp('maximum number of iteration is reached');
    end
    
    % find the optimal match in each train
    clear indc idsm;
    for k = 1:N
        if nS(k) >= nm
            [warp_dis, mf1, mg1] = match_sub(S{k}, s, T, 2, 1e-3);
            indc{k} = mf1';
            dism(k) = warp_dis;
        end
        if nS(k) < nm
            if nS(k) == 0;
                indc{k} = [];
                dism(k) = 0;
            else
                [warp_dis, mf1, mg1] = match_sub(S{k}, s, T, 2, 1e-3);
                indc{k} = mg1';
                dism(k) = warp_dis;
            end
        end
        dist(h) = sum(dism);
    end
    
    % reestimate the mean spike train position
    clear st isi;
    for k = 1:N
        if nS(k) >= nm
            st =  S{k}(indc{k});
            isi(k,:) = diff([0 st T], 1, 2);
        end
        if nS(k) < nm
            st = interp1([0 s(indc{k}) T], [0 S{k} T], s);
            isi(k,:) = diff([0 st T]);
        end
    end
    
    isi_mean = sum(sqrt(isi),1).^2/sum((sum(sqrt(isi),1)).^2)*T;
    s_new = cumsum(isi_mean(1:end-1));
    ss(h+1,:) = s_new;
    
    dis(h) = sum(sum((sqrt(isi) - sqrt(ones(N,1)*isi_mean)).^2));
    
    if norm(s-s_new) < 1e-2
        break;
    else
        s = s_new;
    end
end
means = s;


