% generate spike trains from an inhomogeneous Poisson process

clear; close all;

T = 2;  % total time 2 sec
binsize = 0.001;  % bin size 1ms
N = T/binsize;
t = binsize:binsize:T;


%%%% general spike trains based on poisson models
rho(1,:) =  16*(normpdf(t, 0.8, 0.15));
rho(2,:) =  16*(normpdf(t, 1.2, 0.3));

C = size(rho,1); 

K = 20;
rand('seed', 30);
for c = 1:C
    S0{c} = double(rand(K,N)<ones(K,1)*rho(c,:)*binsize);
end

for c = 1:C
    for k = 1:K
        S{k+(c-1)*K} = find(S0{c}(k,:))/N*T;
        nS(k+(c-1)*K) = length(S{k+(c-1)*k});
    end
end

figure(1); clf;
cr = 'brgc';
subplot(3,1,1);
for c = 1:C    
    hold on;
    plot(t, rho(c,:), 'color', cr(c), 'linewidth', 2);
end
% xlabel('time (sec)', 'fontsize', 18);
ylabel('\rho', 'fontsize', 14);
axis([0 2 0 45]);
set(gca, 'xtick', [], 'ytick', [0 40], 'fontsize', 14);

subplot(3,1,[2 3]);
for c = 1:C
    for i = 1:20
        hold on;
        plot(ones(2,1)*S{i+(c-1)*K}, [21-i, 20-i]+20*(C-c), cr(c), 'linewidth', 1);
    end
end
xlabel('time (sec)', 'fontsize', 18);
axis([0 2 0 40]);
set(gca, 'ytick', 1:10:31, 'yticklabel', 40:-10:10, 'fontsize', 14);

save ipp_data T S nS;  

